@@ -1,8 +1,11 @@
 use std::error::Error as StdError;
 use std::fmt;
+use std::io::Error as IoError;
+use std::net::AddrParseError;
+use std::rc::Rc;
 use std::result::Result as StdResult;
+use std::str::Utf8Error;
 use std::string::String as StdString;
-use std::sync::Arc;
 
 /// Error type returned by `mlua` methods.
 #[derive(Debug, Clone)]
@@ -115,7 +118,7 @@ pub enum Error {
         /// Lua call stack backtrace.
         traceback: StdString,
         /// Original error returned by the Rust code.
-        cause: Arc<Error>,
+        cause: Rc<Error>,
     },
     /// A custom error.
     ///
@@ -124,7 +127,7 @@ pub enum Error {
     /// Returning `Err(ExternalError(...))` from a Rust callback will raise the error as a Lua
     /// error. The Rust code that originally invoked the Lua code then receives a `CallbackError`,
     /// from which the original error (and a stack traceback) can be recovered.
-    ExternalError(Arc<dyn StdError + Send + Sync>),
+    ExternalError(Rc<dyn StdError>),
 }
 
 /// A specialized `Result` type used by `mlua`'s API.
@@ -184,10 +187,10 @@ impl fmt::Display for Error {
             Error::MismatchedRegistryKey => {
                 write!(fmt, "RegistryKey used from different Lua state")
             }
-            Error::CallbackError { ref traceback, ref cause } => {
-                write!(fmt, "callback error: {}: {}", cause, traceback)
+            Error::CallbackError { ref traceback, .. } => {
+                write!(fmt, "callback error: {}", traceback)
             }
-            Error::ExternalError(ref err) => write!(fmt, "external error: {}", err),
+            Error::ExternalError(ref err) => write!(fmt, "{}", err),
         }
     }
 }
@@ -196,14 +199,14 @@ impl StdError for Error {
     fn source(&self) -> Option<&(dyn StdError + 'static)> {
         match *self {
             Error::CallbackError { ref cause, .. } => Some(cause.as_ref()),
-            Error::ExternalError(ref err) => Some(err.as_ref()),
+            Error::ExternalError(ref err) => err.source(),
             _ => None,
         }
     }
 }
 
 impl Error {
-    pub fn external<T: Into<Box<dyn StdError + Send + Sync>>>(err: T) -> Error {
+    pub fn external<T: Into<Box<dyn StdError>>>(err: T) -> Error {
         Error::ExternalError(err.into().into())
     }
 }
@@ -214,7 +217,7 @@ pub trait ExternalError {
 
 impl<E> ExternalError for E
 where
-    E: Into<Box<dyn StdError + Send + Sync>>,
+    E: Into<Box<dyn StdError>>,
 {
     fn to_lua_err(self) -> Error {
         Error::external(self)
@@ -233,3 +236,21 @@ where
         self.map_err(|e| e.to_lua_err())
     }
 }
+
+impl std::convert::From<AddrParseError> for Error {
+    fn from(err: AddrParseError) -> Self {
+        Error::external(err)
+    }
+}
+
+impl std::convert::From<IoError> for Error {
+    fn from(err: IoError) -> Self {
+        Error::external(err)
+    }
+}
+
+impl std::convert::From<Utf8Error> for Error {
+    fn from(err: Utf8Error) -> Self {
+        Error::external(err)
+    }
+}
