@@ -1,10 +1,4 @@
-#![allow(unused_imports)]
-
-use std::{string::String as StdString, time::Duration};
-
-use futures_executor::block_on;
-
-use mlua::{Error, Function, Lua, Result, String, Thread};
+use mlua::{Function, Lua, Result, String};
 
 #[test]
 fn test_function() -> Result<()> {
@@ -81,34 +75,3 @@ fn test_rust_function() -> Result<()> {
 
     Ok(())
 }
-
-#[cfg(feature = "async")]
-#[tokio::test]
-async fn test_async_function() -> Result<()> {
-    let lua = Lua::new();
-
-    let f = lua.create_async_function(move |_lua, n: u64| async move {
-        futures_timer::Delay::new(Duration::from_secs(n)).await;
-        Ok("hello")
-    })?;
-    lua.globals().set("rust_async_sleep", f)?;
-
-    let thread = lua
-        .load(
-            r#"
-            coroutine.create(function ()
-                ret = rust_async_sleep(1)
-                assert(ret == "hello")
-                coroutine.yield()
-                return "world"
-            end)
-        "#,
-        )
-        .eval::<Thread>()?;
-
-    let fut = thread.into_async(());
-    let ret: StdString = fut.await?;
-    assert_eq!(ret, "world");
-
-    Ok(())
-}
